// - - - - -
// DMXBridge.ino:
// Application for sending DMX over an 2.4 GHz wireless link
// using Arduinos, DMX shields and nRF24L01+ modules.
// Copyright (c) 2013 by Matthias Hertel, http://www.mathertel.de
// This work is licensed under a BSD style license. See http://www.mathertel.de/License.aspx
//
// Documentation and samples are available at http://www.mathertel.de/Arduino
// 01.06.2013 creation of the DMXBridge project.
// 28.06.2013 prototype is running.
// 01.07.2013 creation of test sketches.
//
// This is sketch is part of the DMXBridge project for sending DMX values
// over a 2.4 GHz wireless link using Arduinos, DMX shields and nRF24L01+ modules.
//
// This is an example sends and receives 3 values (non DMX)
// over the link for testing sender and receiver implementations
// and various options.
// The values can be watched at the PWM outputs of the Arduino Board.

// Prerequisites:
// * Import the DMXSerial library into the Sketches/libraries folder.
//   http://www.mathertel.de/Arduino/DMXSerial.aspx
// * Import the RF24 library into the Sketches/libraries folder.
//   https://github.com/gcopeland/RF24 (use the [Download ZIP] button on the right)
// * compile the Sketch to see if everything is there.
// - - - - -

#include <DMXSerial.h>

#include <SPI.h>
#include <RF24.h>

// Set up role.  This sketch uses the same software for all the nodes.
// Change the following definition before compiling/uploading to the nodes.
#define DMXBIDGE_SENDER 0 // set to 1 for SENDER, set to 0 for RECEIVER


// Use LEDs to check some of the DMX values
// set this definition to 1 to enable sending the values 1-3 to the PWM pins.

#define LEDTEST 1

#if LEDTEST
#define LEDCOUNT 3
const uint8_t led_pins[LEDCOUNT] = {9, 6, 5};
#endif


// Hardware and Topology configuration

// Set up nRF24L01 radio on SPI bus plus pins 7 & 8
RF24 radio(7,8);

// Single radio pipe address for the 2 nodes to communicate.
const uint64_t PIPE = 0xE8E8F0F0E1LL;


// PAYLOAD TYPE
// structure of the payload / a transmitted package
struct payload_t {
  uint8_t messageNr;
  uint8_t start8;        // dmx start address of data = start8 * 8
  uint8_t dmxValues[8];  // dmxdata[8 * start8 + 1] .. dmxdata[8 * start8 + 8]
}; // payload_t


uint8_t dmxSentData[DMXSERIAL_MAX+1];
int     fastIdx; // last checked index position using the fast mode.
int     slowIdx; // last checked index position using the slow mode.
unsigned long nextSlow = 0; // 
uint8_t messageNr = 0;


// The role of the current running sketch
boolean isReceiver = true;

// setup the Board and nRF24L01
void setup(void)
{
  // Print preamble
  // Serial.begin(57600);
  // Serial.println("DMXBRIDGE RF24 DMX Transceiver");

  // Setup and configure rf radio
  radio.begin();

  radio.setChannel(101);   // use channel 101
  radio.setDataRate(RF24_2MBPS); // use fastest transfer speed
  radio.setAutoAck(false); // use broadcast mode

#if LEDTEST
  for (int i = 0; i < LEDCOUNT; i++) {
    pinMode(led_pins[i],OUTPUT);
    analogWrite(led_pins[i], 0); 
  } // for
#endif

#if (DMXBIDGE_SENDER)
  // receive DMX values...
  DMXSerial.init(DMXReceiver);

  // ... and send them into the air.
  radio.openWritingPipe(PIPE);

  fastIdx = 1;
  slowIdx = 1;
  nextSlow = millis() + 2000;

#else
  // receive NRF24L01 data ...
  radio.openReadingPipe(1,PIPE);
  radio.startListening();

  // ... and send them as DMX packages.
  DMXSerial.init(DMXController);
#endif

} // setup()


// Send a part of the dmx values into the air.
// Send the package that includes the DMX channel idx.
// Take the values from the DMX input buffer and remember them in the dmxSentData array.
// Return the index of the next DMX channel not sent.
int SendBuffer (int idx)
{
  messageNr++;
  payload_t payload;
  
  int p = ((idx-1) % DMXSERIAL_MAX) / 8; // part to be sent
  int i = p*8 + 1;
  
  // populate payload
  payload.start8 = p;
  payload.messageNr = messageNr;
  for (int n = 0; n < 8; n++) {
    payload.dmxValues[n] = dmxSentData[i] = DMXSerial.read(i);
    i++;
  } // for
  radio.write(&payload, sizeof(payload), true);
  return (i);
} // SendBuffer()


void loop(void)
{
  unsigned long now = millis();
  static uint8_t message_nr = 0;

  static uint32_t message_count = 0;
  payload_t payload;

#if (DMXBIDGE_SENDER)
  // DMX Sender role.  Receive each packet, dump it out, add ack payload for next time
  if (now > nextSlow) {
    // Send a package with the current values
    // even if they have changed or not every 2 seconds.
    slowIdx = SendBuffer(slowIdx);
    nextSlow = now + 2000;
    
  } else {      
    // Send the next package with changed values.
    // max. check next 20 values
    for (int n = 0; n <= 20; n++) {
      fastIdx = (fastIdx % DMXSERIAL_MAX) + 1;
      if (DMXSerial.read(fastIdx) != dmxSentData[fastIdx]) {
        fastIdx = SendBuffer(fastIdx);
        break; // so good for now.
      } // if
    } // for
  } // if

#else
  // Receiver role. Receive 

  // if there is data ready
  if (radio.available()) {

    // Fetch the payload, and see if this was ok.
    bool done = radio.read(&payload, sizeof(payload));

    if (done) {
      int i = payload.start8 * 8 + 1;
  
      for (int n = 0; n < 8; n++, i++)
        DMXSerial.write(i, payload.dmxValues[n]);
    } // if
  } // if

#endif

#if LEDTEST
  // for test purpose: set PWM outputs to DMX channel 1..3
  for (int i = 0; i < LEDCOUNT; i++) {
    analogWrite(led_pins[i], DMXSerial.read(7+i)); 
  } // for
#endif


} // loop()

// The End.
