// - - - - -
// DMXBridge_TestSender.ino: A DMXBridge sender test sketch.
// Copyright (c) 2013 by Matthias Hertel, http://www.mathertel.de
// This work is licensed under a BSD style license. See http://www.mathertel.de/License.aspx
// 
// Documentation and samples are available at http://www.mathertel.de/Arduino
// 01.06.2013 creation of the DMXBridge project.
// 28.06.2013 prototype is running.
// 01.07.2013 creation of test sketches.
//
// This is sketch is part of the DMXBridge project for sending DMX values
// over a 2.4 GHz wireless link using Arduinos, DMX shields and nRF24L01+ modules.

// This is an example sends 60 DMX values for 20 RGB receivers
// by using a more complex RGB based pattern of colors.
// There is no need for a real DMX source for testing the DMXBridge.
// The values for channel 1-3 can be watched at the PWM outputs of the Arduino Board.

// Prerequisites:
// * Import the DMXSerial library into the Sketches/libraries folder.
//   http://www.mathertel.de/Arduino/DMXSerial.aspx
// * Import the RF24 library into the Sketches/libraries folder.
//   https://github.com/gcopeland/RF24 (use the [Download ZIP] button on the right)
// * compile the Sketch to see if everything is there.
// - - - - -

#include <SPI.h>
#include <RF24.h>

#define ACKPAY 0  // include acknowledge date 
#define ACKAUTO 0 // set auto acknowledge
#define ACKNONE 1 // use multicast - no acknowledge

// Hardware and Topology configuration

// Set up nRF24L01 radio on SPI bus plus pins 7 & 8
RF24 radio(7,8);

// Single radio pipe address for the nodes to communicate.
const uint64_t PIPE = 0xE8E8F0F0E1LL;

// PAYLOAD TYPE
// structure of the payload / a transmitted package
struct payload_t {
  uint8_t messageNr;
  uint8_t start8;        // dmx start address of data = start8 * 8
  uint8_t dmxValues[8];  // dmxdata[8 * start8 + 1] .. dmxdata[8 * start8 + 8]
}; // payload_t

#define  DMXSERIAL_MAX 512
uint8_t dmxSentData[DMXSERIAL_MAX+1];
int     index; // last start address of payload
uint8_t messageNr = 0;

// see the RGB values 1-3 on PWM outputs.
#define LEDCOUNT 3
const uint8_t led_pins[LEDCOUNT] = {9, 6, 5};

// setup the Board and nRF24L01
void setup(void)
{
  // Serial.begin(57600);
  // Print preamble
  // Serial.println("RF24 DMX Transceiver");

  // Setup and configure rf radio
  radio.begin();

  radio.setChannel(101);   
  radio.setDataRate(RF24_2MBPS); 
  radio.setAutoAck(false); 

  for (int i = 0; i < LEDCOUNT; i++)
    pinMode(led_pins[i], OUTPUT);

  // Open pipes for sending

  radio.openWritingPipe(PIPE);
  index = 1;
}

// set 3 channels to a RGB value with the specified hue (0...764)
void setChannelRGB(int channel, int hue) {
  hue = hue % 765;
  if (hue < 256) {
    /// blue to red
    dmxSentData[channel  ] = hue;
    dmxSentData[channel+1] = 0;
    dmxSentData[channel+2] = 255-hue;

  } else if (hue < 511) {
    /// red to green
    dmxSentData[channel  ] = 255 - (hue-255);
    dmxSentData[channel+1] = hue-255;
    dmxSentData[channel+2] = 0;

  } else {
    /// green to blue
    dmxSentData[channel  ] = 0;
    dmxSentData[channel+1] = 255 - (hue-510);
    dmxSentData[channel+2] = hue-510;
  } // if
} // setChannelRGB()


// Send a part of the dmx values into the air.
// return the next index not sent.
int SendBuffer (int idx)
{
  messageNr++;
  payload_t payload;
  
  int p = ((idx-1) % DMXSERIAL_MAX) / 8; // part to be sent
  int i = p*8 + 1;
  
  // populate payload
  payload.start8 = p;
  payload.messageNr = messageNr;
  for (int n = 0; n < 8; n++) {
    payload.dmxValues[n] = dmxSentData[i] = dmxSentData[i];
    i++;
  } // for
  radio.write(&payload, sizeof(payload), true);
  return (i);
} // SendBuffer()


void loop(void)
{
  unsigned long now = millis();
  static uint8_t message_nr = 0;

  // Simulate some DMX input values for testing purpose
  // create some DMX test values: 5 RGB channels
  int alpha = (now / 12) % 765;

  // uncomment this line to have a scenario where DMX values are changed ~ 2 times the second 
  // alpha = (alpha / 64) * 64;
  // uncomment this line to have a scenario where DMX values are changed every 5 seconds 
  // alpha &= 0xFF00;

  for (int n = 0; n < 20; n++) {
    setChannelRGB(n*3+1, alpha + n*32);
  } // for

  index = SendBuffer(index);

  // for test purpose: set PWM outputs to DMX channel 1..3
  for (int i = 0; i < LEDCOUNT; i++) {
    analogWrite(led_pins[i], dmxSentData[1+i]); 
  } // for

} // loop()

// The End.

