// - - - - -
// NRF24_RGBLed.ino: Sample application for sending single RGB values
// over a NRF24P01 transceiver.
// Copyright (c) 2013 by Matthias Hertel, http://www.mathertel.de
// This work is licensed under a BSD style license. See http://www.mathertel.de/License.aspx
//
// Documentation and samples are available at http://www.mathertel.de/Arduino
// 01.06.2013 creation of the DMXBridge project.
// 28.06.2013 prototype is running.
// 01.07.2013 creation of test sketches.
//
// This is sketch is part of the DMXBridge project for sending DMX values
// over a 2.4 GHz wireless link using Arduinos, DMX shields and nRF24L01+ modules.
//
// This is an example sends and receives 3 values (non DMX)
// over the link for testing sender and receiver implementations
// and various options.
// The values can be watched at the PWM outputs of the Arduino Board.

// Prerequisites:
// * Import the RF24 library into the Sketches/libraries folder.
//   https://github.com/gcopeland/RF24 (use the [Download ZIP] button on the right)
// * compile the Sketch to see if everything is there.
// - - - - -

#include <SPI.h>
#include <RF24.h>

#define ACKPAY 0  // include acknowledge date 
#define ACKAUTO 0 // set auto acknowledge
#define ACKNONE 1 // use multicast - no acknowledge

// Set up role.  This sketch uses the same software for all the nodes.
// Change the following definition before compiling/uploading to the nodes.
#define ONELED_SENDER 0 // set to 1 for SENDER, set to 0 for RECEIVER

// Hardware and Topology configuration

// Set up nRF24L01 radio on SPI bus plus pins 7 & 8
RF24 radio(7,8);

// Single radio pipe address for the 2 nodes to communicate.
const uint64_t PIPE = 0xE8E8F0F0E1LL;

#define LEDCOUNT 3

// PAYLOAD TYPE
// structure of the payload / a transmitted package
struct payload_t {
  uint8_t messageNr;
  uint8_t start8;        // dmx start address of data = start8 * 8
  uint8_t dmxValues[8];  // dmxdata[8 * start8 + 1] .. dmxdata[8 * start8 + 8]
}; // payload_t


// PWM pins to be used
const uint8_t led_pins[LEDCOUNT] = {9,6,5};

// local buffer for the rgbValues
uint8_t rgbValues[LEDCOUNT];

// setup everything
void setup(void)
{
  // Print preamble
  Serial.begin(57600);
  Serial.println("NRF24_RGBLed Example");

  // Setup and configure rf radio
  radio.begin();
  radio.setChannel(101);   
  radio.setDataRate(RF24_2MBPS); // RF24_250KBPS, RF24_1MBPS, RF24_2MBPS

#if ACKPAY
  // We will be using the Ack Payload feature, so please enable it
  radio.enableAckPayload();
#endif

#if ACKAUTO
  radio.setAutoAck(true); 
  radio.setPayloadSize(sizeof(payload_t)); // define tamanho máximo da var a transmitir
  radio.setRetries(15,15);               //define n de tentativas e tempo entre elas (ainda falta perceber como e que isto esta a funcionar)
#endif

#if ACKNONE
  radio.setAutoAck(false); 
#endif

  // Open pipes
  // This simple sketch opens a single pipe for communication.
  // The sender writes on it the receiver listens.

#if (ONELED_SENDER)
  radio.openWritingPipe(PIPE);

#else
  radio.openReadingPipe(1,PIPE);
  radio.startListening();
#endif

  // Enable LED PWM output
  for (int i = 0; i < LEDCOUNT; i++)
    pinMode(led_pins[i],OUTPUT);
} // setup()


// repeat sending and receiving
void loop(void)
{
  unsigned long now;
  static uint8_t message_nr = 0;
  static uint32_t message_count = 0;

  payload_t payload;

#if (ONELED_SENDER)
  // Sender role.  Repeat sending RGB values

  // payload.messageNr is used for sending the message number.
  // That helps identifying dropped packages.
  payload.messageNr = message_nr++;
  payload.start8 = 0;

  // Take the time, and create a rainbow color.
  int alpha = (millis() / 12) % 765;
  if (alpha < 256) {
    rgbValues[0] = (alpha-0);
    rgbValues[1] = 0;
    rgbValues[2] = 255-(alpha-0);

  } else if (alpha < 511) {
    rgbValues[0] = 255 - (alpha-255);
    rgbValues[1] = (alpha-255);
    rgbValues[2] = 0;

  } else {
    rgbValues[0] = 0;
    rgbValues[1] = 255 - (alpha-510);
    rgbValues[2] = (alpha-510);
  } // if

  payload.dmxValues[0] = rgbValues[0];
  payload.dmxValues[1] = rgbValues[1];
  payload.dmxValues[2] = rgbValues[2];

  for (int i = 3; i < 8; i++)
    payload.dmxValues[i] = 0;

  /*
    Serial.print("Sending: ");  // Serial.print(payload.messageNr);
    Serial.print(alpha); 
     Serial.print(" ");
     Serial.print(rgbValues[0]); 
     Serial.print(" ");
     Serial.print(rgbValues[1]); 
     Serial.print(" ");
     Serial.print(rgbValues[2]); 
     Serial.print(" ");
     Serial.print(payload.messageNr);
  */

  // now = - micros();
  // write out and wait for Acknowledge
  radio.write(&payload, sizeof(payload), true);
  // now += micros();
  // Serial.print("using...");  Serial.print(now);


  if (radio.isAckPayloadAvailable()) {
    radio.read(&message_count, sizeof(message_count));
    printf("Ack: [%lu] ", message_count);
  }
  // Serial.print(" OK\n\r");

  // Try again soon
  //    delay(100);

#else
  // Receiver role.

  // if there is data ready
  if ( radio.available() ) {
    // fetch the payload, and see if this was the last one.
    bool done = radio.read(&payload, sizeof(payload));

    if (payload.start8 == 0) {
      // ignore all packages beyond channels 1-8
      // save the values to local buffer
      for (int i = 0; i < LEDCOUNT; i++)
        rgbValues[i] = payload.dmxValues[i];
    } // if
    
    // check message-number and identify dropped packages.
    message_nr++;
    if (payload.messageNr != message_nr) {
      Serial.print("missing packages from "); Serial.print(message_nr);
      Serial.print(" to "); Serial.println(payload.messageNr-1);
    } // if
    message_nr = payload.messageNr;

#if ACKPAY
    // Add an ack packet for the next time around.  This is a simple
    // packet counter
    radio.writeAckPayload( 1, &message_count, sizeof(message_count) );
    ++message_count;
#endif

  } // if available
#endif
  
  // set the PWM output pins for testing purpose.
  for (int i = 0; i < LEDCOUNT; i++) {
    analogWrite(led_pins[i], rgbValues[i]); 
  } // for

} // loop()

// The End




